@@ -17,7 +17,6 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import print_function
 import argparse
 import atexit
 import collections
@@ -59,7 +58,7 @@ signal.signal(signal.SIGINT, sigint_handler)
 
 # Constants
 
-_VERSION_ = '2.5.1'
+_VERSION_ = '2.6'
 
 COLORMAP = {k: '\x1b[%sm' % v for k, v in {
     'a': '30', 'b': '31', 'c': '32', 'd': '33',
@@ -966,7 +965,7 @@ class GoogleParser(html.parser.HTMLParser):
     #
     # Ignore List
     #
-    # - As good as our result criterions might be, sometimes results of
+    # - As good as our result criteria might be, sometimes results of
     #   dubious value (usually from Google's value-add features) slip
     #   through. The "People also ask" feature is a good example of this
     #   type (a sample query is "VPN"; see screenshot
@@ -988,6 +987,42 @@ class GoogleParser(html.parser.HTMLParser):
     #   title, etc.), and the scope is switched to 'ignore'. All
     #   descendants of the tag are ignored. When the corresponding end
     #   tag is finally reach, the former scope is restored.
+    #
+    #
+    # User Agent disabled (differences)
+    #
+    #   1. For Google News results, <div class="g"> is followed by <table> tag
+    #       <div class="g">
+    #           <table>
+    #
+    #   2. File mime type follows <div class="g">
+    #       <div class="g"><span style="float:left"><span class="mime">[PDF]</span>&nbsp;</span>
+    #
+    #   3. News metadata (source and time) comes within a single tag
+    #       <div class="slp"><span class="f">Reuters - 3 hours ago</span>
+    #
+    #   4. URLs are wrapped
+    #       <a href="/url?q=http://...&sa=...">
+    #
+    #   5. URLs are quoted
+    #       'https://vk.com/doc206446660_429188746%3Fhash%3D6097a8b0a41185cb90%26dl%3D03c63c1be5c02e8620'
+    #
+    #   6. Google Services links are returned as regular results,
+    #      start with '/search?q=' but no following 'http' or 'https'
+    #       <div class="g">
+    #           <div>
+    #               <h3 class="r"><a href="/search?q=india&...&sa=...">News for <b>india</b></a></h3>
+    #
+    #   7. YouTube specific results are returned within (.e.g search - '3 hours youtube')
+    #       <table class="ts">
+    #
+    #      TODO: In addition, the text is returned within multi-level <span> tags:
+    #       <span class="st">
+    #           <span class="f"><span class="nobr">10 Jun 2014</span> - <span class="nobr">179 min</span> -
+    #               <span class="nobr">Uploaded by Meditation Relax Music</span>
+    #           </span>
+    #           <br><b>3 HOURS Best Relaxing Music</b> &#39;Romantic <b>Piano</b>&quot; Background <b>Music</b> for Stress ... 3:03 <b>...</b>
+    #       </span>
 
     def __init__(self, news=False):
         html.parser.HTMLParser.__init__(self)
@@ -1038,7 +1073,7 @@ class GoogleParser(html.parser.HTMLParser):
             self.set_handlers_to('result')
             return 'result'
 
-        # Ommitted results
+        # Omitted results
         if tag == 'p' and attrs.get('id') == 'ofr':
             self.filtered = True
 
@@ -1048,6 +1083,10 @@ class GoogleParser(html.parser.HTMLParser):
 
     @annotate_tag
     def result_start(self, tag, attrs):
+        if '' == USER_AGENT and tag == 'span' and 'mime' in self.classes(attrs):
+            self.start_populating_textbuf()
+            return 'title_filetype'
+
         if not self.title_registered and tag == 'h3' and 'r' in self.classes(attrs):
             self.set_handlers_to('title')
             return 'title'
@@ -1057,13 +1096,18 @@ class GoogleParser(html.parser.HTMLParser):
             return 'abstract'
 
         if not self.sitelinks and tag == 'table':
-            self.set_handlers_to('sitelink_table')
-            return 'sitelink_table'
+            if '' != USER_AGENT or (not self.news and 'ts' not in self.classes(attrs)):
+                self.set_handlers_to('sitelink_table')
+                return 'sitelink_table'
 
         if self.news:
             if not self.metadata_registered and tag == 'div' and 'slp' in self.classes(attrs):
                 # Change metadata field separator from '-' to ', ' for better appearance
-                self.start_populating_textbuf(lambda text: ', ' if text == '-' else text)
+                if '' != USER_AGENT:
+                    self.start_populating_textbuf(lambda text: ', ' if text == '-' else text)
+                else:
+                    self.start_populating_textbuf(lambda text:
+                                              text.replace(' -', ',', 1) if ' - ' in text else text)
                 return 'news_metadata'
 
             if not self.abstract_registered and tag == 'div' and 'st' in self.classes(attrs):
@@ -1091,12 +1135,22 @@ class GoogleParser(html.parser.HTMLParser):
 
     @annotate_tag
     def title_start(self, tag, attrs):
-        if tag == 'span':
+        if '' != USER_AGENT and tag == 'span':
             # Print a space after the filetype indicator
             self.start_populating_textbuf(lambda text: text + ' ')
             return 'title_filetype'
         if tag == 'a' and 'href' in attrs:
+            # Skip 'News for', 'Images for' search links
+            if attrs['href'].startswith('/search'):
+                return
+
             self.url = attrs['href']
+            try:
+                start = self.url.index('?q=') + len('?q=')
+                end = self.url.index('&sa=', start)
+                self.url = urllib.parse.unquote_plus(self.url[start:end])
+            except ValueError:
+                pass
             self.start_populating_textbuf()
             return 'title_link'
 
@@ -1162,6 +1216,12 @@ class GoogleParser(html.parser.HTMLParser):
     def sitelink_title_start(self, tag, attrs):
         if tag == 'a' and 'href' in attrs:
             self.current_sitelink.url = attrs['href']
+            try:
+                start = self.current_sitelink.url.index('?q=') + len('?q=')
+                end = self.current_sitelink.url.index('&sa=', start)
+                self.current_sitelink.url = urllib.parse.unquote_plus(self.current_sitelink.url[start:end])
+            except ValueError:
+                pass
             self.start_populating_textbuf()
             return 'sitelink_title_link'
 
@@ -1355,7 +1415,7 @@ class Result(object):
         if columns > indent + 1:
             # Try to fill to columns
             fillwidth = columns - indent - 1
-            for line in textwrap.wrap(abstract, width=fillwidth):
+            for line in textwrap.wrap(abstract.replace('\n', ''), width=fillwidth):
                 print('%s%s' % (' ' * indent, line))
             print('')
         else:
@@ -1430,7 +1490,7 @@ def no_argument(method):
         if arg:
             method_name = arg.__name__
             command_name = method_name[3:] if method_name.startswith('do_') else method_name
-            logger.warn("Argument to the '%s' command ignored.", command_name)
+            logger.warning("Argument to the '%s' command ignored.", command_name)
         method(self)
 
     return enforced_method
@@ -1855,6 +1915,8 @@ def parse_args(args=None, namespace=None):
            help='search a site using Google')
     addarg('-p', '--proxy', dest='proxy',
            help='tunnel traffic through an HTTPS proxy (HOST:PORT)')
+    addarg('--noua', dest='noua', action='store_true',
+           help='disable user agent')
     addarg('--json', dest='json', action='store_true',
            help='output in JSON format; implies --noprompt')
     addarg('--np', '--noprompt', dest='noninteractive', action='store_true',
@@ -1867,13 +1929,14 @@ def parse_args(args=None, namespace=None):
 
 
 def main():
-    logger.debug('Version %s', _VERSION_)
+    global USER_AGENT
 
     opts = parse_args()
 
     # Set logging level
     if opts.debug:
         logger.setLevel(logging.DEBUG)
+        logger.debug('Version %s', _VERSION_)
 
     # Set colors
     if opts.colorize:
@@ -1883,6 +1946,9 @@ def main():
     Result.colors = colors
     GooglerCmd.colors = colors
 
+    if opts.noua:
+        USER_AGENT = ''
+
     try:
         repl = GooglerCmd(opts)
 
@@ -1911,3 +1977,4 @@ def main():
 
 if __name__ == '__main__':
     main()
+    atexit.register(logging.shutdown)
